// d_crgrph.cpp : implementation file
//

#include "stdafx.h"
#include "grphdemo.h"
#include "d_crgrph.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif

BOOL Flip(double prob)
{
	if (prob >= 1.0)
		return TRUE ;
	if (prob == 0)
		return FALSE ;
	const double max = ((double)RAND_MAX) + 1.0 ;
	return (rand()/max <= prob) ;
}

/////////////////////////////////////////////////////////////////////////////
// CCreateGraphDlg dialog

CCreateGraphDlg::CCreateGraphDlg(CWordMatrix &graph, CWnd* pParent /*= NULL*/)
	: CDialog(CCreateGraphDlg::IDD, pParent),  
	  m_Graph(graph) 
{
	//{{AFX_DATA_INIT(CCreateGraphDlg)
	m_NumNodes = 0;
	m_ConnectProb = 0;
	//}}AFX_DATA_INIT
}

void CCreateGraphDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CCreateGraphDlg)
	DDX_Control(pDX, IDC_EDIT_CONNECT_PROB, m_EditConnectProb);
	DDX_Control(pDX, IDC_LIST_TO_NODES, m_ListToNodes);
	DDX_Control(pDX, IDC_LIST_FROM_NODES, m_ListFromNodes);
	DDX_Control(pDX, IDC_EDIT_NUM_NODES, m_EditNumNodes);
	DDX_Text(pDX, IDC_EDIT_NUM_NODES, m_NumNodes);
	DDV_MinMaxInt(pDX, m_NumNodes, 0, MAX_NODES);
	DDX_Text(pDX, IDC_EDIT_CONNECT_PROB, m_ConnectProb);
	DDV_MinMaxDouble(pDX, m_ConnectProb, 0., 1.);
	//}}AFX_DATA_MAP
}

BEGIN_MESSAGE_MAP(CCreateGraphDlg, CDialog)
	//{{AFX_MSG_MAP(CCreateGraphDlg)
	ON_BN_CLICKED(IDC_BUTTON_ADD_NODE, OnClickedButtonAddNode)
	ON_BN_CLICKED(IDC_BUTTON_CONNECT, OnClickedButtonConnect)
	ON_BN_CLICKED(IDC_BUTTON_DEL_ALL_CONNECT, OnClickedButtonDelAllConnect)
	ON_BN_CLICKED(IDC_BUTTON_DEL_NODE, OnClickedButtonDelNode)
	ON_BN_CLICKED(IDC_BUTTON_DELETE_CONNECT, OnClickedButtonDeleteConnect)
	ON_BN_CLICKED(IDC_BUTTON_RND_CONNECT, OnClickedButtonRndConnect)
	ON_EN_KILLFOCUS(IDC_EDIT_NUM_NODES, OnKillfocusEditNumNodes)
	ON_LBN_SELCHANGE(IDC_LIST_FROM_NODES, OnSelchangeListFromNodes)
	ON_LBN_SELCHANGE(IDC_LIST_TO_NODES, OnSelchangeListToNodes)
	ON_EN_KILLFOCUS(IDC_EDIT_CONNECT_PROB, OnKillfocusEditConnectProb)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CCreateGraphDlg message handlers

void CCreateGraphDlg::OnClickedButtonAddNode()
{
	// TODO: Add your control notification handler code here
	char buffer[10] ;
	int oldNum =  m_NumNodes ;
	m_EditNumNodes.GetWindowText(buffer,9) ;
	m_NumNodes = atoi(buffer) ;
	if (m_NumNodes > MAX_NODES) {
		m_NumNodes = MAX_NODES ;
		m_EditNumNodes.SetWindowText("50") ;
		}	
	if (m_NumNodes < 0) {
		m_NumNodes = 0 ;	
		m_EditNumNodes.SetWindowText("0") ;
		}
	int diff = m_NumNodes - oldNum ;
	if (diff < 0)
		DelNodes(diff*-1) ;
	else
	if (diff > 0)
		AddNodes(diff) ;
}

void CCreateGraphDlg::OnClickedButtonConnect()
{
	// TODO: Add your control notification handler code here
	int from = m_ListFromNodes.GetCurSel() ;
	if (from == LB_ERR)
		return ;
		
	//first delete previous connections
	int numNodes = m_Graph.GetNumCols() ;
	for (int i=0;i<numNodes;i++)
		m_Graph[from][i] = FALSE ;
		
	//Now establish new ones 
	int ToNodes[MAX_NODES] ;
	int numToNodes = m_ListToNodes.GetSelItems(MAX_NODES,ToNodes) ;
	for (i=0;i<numToNodes;i++)
		if (from != i)
			m_Graph[from][ToNodes[i]] = TRUE ;	
}

void CCreateGraphDlg::OnClickedButtonDelAllConnect()
{
	// TODO: Add your control notification handler code here
	int numNodes = m_Graph.GetNumCols() ;
	for (int i=0;i<numNodes;i++)
		for (int j=0;j<numNodes;j++)
			m_Graph[i][j] = FALSE ;
	m_ListToNodes.SetSel(-1,FALSE) ;
}

void CCreateGraphDlg::OnClickedButtonDelNode()
{
	// TODO: Add your control notification handler code here
	m_NumNodes--;
	if ( m_NumNodes < 0)
		m_NumNodes = 0 ;
	char buffer[10] ;
	sprintf(buffer,"%d",m_NumNodes) ;
	m_EditNumNodes.SetWindowText(buffer) ;
	DelNodes(1) ;
}

void CCreateGraphDlg::OnClickedButtonDeleteConnect()
{
	// TODO: Add your control notification handler code here
	int from = m_ListFromNodes.GetCurSel() ;
	if (from == LB_ERR)
		return ;
		
	int numNodes = m_Graph.GetNumCols() ;
	for (int i=0;i<numNodes;i++)
		m_Graph[from][i] = FALSE ;
	ShowConnections(from) ;
}

void CCreateGraphDlg::OnClickedButtonRndConnect()
{
	// TODO: Add your control notification handler code here
	int numNodes = m_Graph.GetNumCols() ;
	for (int row = 0 ; row < numNodes; row++)
		for (int col = 0 ; col < numNodes; col++)
			if (row != col)
				m_Graph[row][col] = Flip(m_ConnectProb) ;	
				
	int from = m_ListFromNodes.GetCurSel() ;
	if (from == LB_ERR)
		return ;
	ShowConnections(from) ;
}

void CCreateGraphDlg::OnKillfocusEditNumNodes()
{
	// TODO: Add your control notification handler code here          
}

void CCreateGraphDlg::OnSelchangeListFromNodes()
{
	// TODO: Add your control notification handler code here   
	m_ListToNodes.SetSel(-1,FALSE) ;
	int sel = m_ListFromNodes.GetCurSel() ;
	if (sel == LB_ERR) 
		return ;
	ShowConnections(sel) ;	
	
}

void CCreateGraphDlg::OnSelchangeListToNodes()
{
	// TODO: Add your control notification handler code here
}

void CCreateGraphDlg::AddNodes(int num) 
{       
	int numNodes = m_ListFromNodes.GetCount() ;
	for(int i=0; i<num;i++) {
		char buffer[20] ;
		sprintf(buffer,"Node%d",numNodes+i) ;
		m_ListFromNodes.AddString(buffer) ;
		m_ListToNodes.AddString(buffer) ;
		}
	
}

void CCreateGraphDlg::DelNodes(int num) 
{       
	int numNodes = m_ListFromNodes.GetCount() ;
	for(int i=0; i<num && numNodes-1-i >= 0;i++) {
		m_ListFromNodes.DeleteString(numNodes-1-i) ;
		m_ListToNodes.DeleteString(numNodes-1-i) ;
		}
}

void CCreateGraphDlg::OnKillfocusEditConnectProb()
{
	// TODO: Add your control notification handler code here   
	char buffer[10] ;
	m_EditConnectProb.GetWindowText(buffer,9) ;
	char * pErr ;
	m_ConnectProb = strtod(buffer,&pErr) ;
	if (*pErr || m_ConnectProb < 0 || m_ConnectProb > 1) {
		MessageBeep(0) ;
		m_EditConnectProb.SetFocus() ;
		return ;
		}
}

void CCreateGraphDlg::ShowConnections(int sel) 
{
	int numNodes = m_Graph.GetNumCols() ;
	for (int i=0;i<numNodes;i++)
		m_ListToNodes.SetSel(i,m_Graph[sel][i]) ;
}
BOOL CCreateGraphDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	
	// TODO: Add extra initialization here
	m_ConnectProb = 0.2 ;
	UpdateData(FALSE) ; //trasfer to dialog controls
	return TRUE;  // return TRUE  unless you set the focus to a control
}
