// grphdvw.cpp : implementation of the CGraphDemoView class
//

#include "stdafx.h"
#include "grphdemo.h"

#include "grphddoc.h"
#include "grphdvw.h"
#include "eos.h"
#include "gdriver.h"
#include "graphga.h"
#include "d_crgrph.h"
#include "d_optim.h"

#ifdef _DEBUG
#undef THIS_FILE
static char BASED_CODE THIS_FILE[] = __FILE__;
#endif
           
CGAGraphDriver * pTheGraphGADriver = NULL ;
           
/////////////////////////////////////////////////////////////////////////////
// CGraphDemoView

IMPLEMENT_DYNCREATE(CGraphDemoView, CView)

BEGIN_MESSAGE_MAP(CGraphDemoView, CView)
	//{{AFX_MSG_MAP(CGraphDemoView)
	ON_COMMAND(ID_GRAPH_CREATE, OnGraphCreate)
	ON_COMMAND(ID_GRAPH_OPTIMIZE, OnGraphOptimize)
	ON_COMMAND(ID_GRAPH_VIEW_OPTIMIZED, OnGraphViewOptimized)
	ON_COMMAND(ID_GRAPH_VIEW_UNOPTIMIZED, OnGraphViewUnoptimized)
	ON_COMMAND(ID_FILE_NEW, OnFileNew)
	ON_UPDATE_COMMAND_UI(ID_GRAPH_CREATE, OnUpdateGraphCreate)
	ON_UPDATE_COMMAND_UI(ID_GRAPH_OPTIMIZE, OnUpdateGraphOptimize)
	ON_COMMAND(ID_GRAPH_STOP, OnGraphStop)
	ON_UPDATE_COMMAND_UI(ID_GRAPH_STOP, OnUpdateGraphStop)
	ON_UPDATE_COMMAND_UI(ID_GRAPH_VIEW_OPTIMIZED, OnUpdateGraphViewOptimized)
	ON_UPDATE_COMMAND_UI(ID_GRAPH_VIEW_UNOPTIMIZED, OnUpdateGraphViewUnoptimized)
	ON_UPDATE_COMMAND_UI(ID_FILE_NEW, OnUpdateFileNew)
	ON_UPDATE_COMMAND_UI(ID_APP_EXIT, OnUpdateAppExit)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CGraphDemoView construction/destruction

CGraphDemoView::CGraphDemoView()                              
	  : m_Graph(MAX_NODES,MAX_NODES) 
{
	// TODO: add construction code here
	m_NumNodes = 0 ; //default number of nodes
	m_View = GV_NONE ;
	m_Evolving = FALSE;
}

CGraphDemoView::~CGraphDemoView()
{
}

/////////////////////////////////////////////////////////////////////////////
// CGraphDemoView drawing

void CGraphDemoView::OnDraw(CDC* pDC)
{
	CGraphDemoDoc* pDoc = GetDocument();

	// TODO: add draw code here
	if (m_View == GV_OPTIMIZED)
		pTheGraphGADriver->DrawOptimized(*pDC) ;
	else	
	if (m_View == GV_UNOPTIMIZED)
		pTheGraphGADriver->DrawUnOptimized(*pDC) ;
}


/////////////////////////////////////////////////////////////////////////////
// CGraphDemoView diagnostics

#ifdef _DEBUG
void CGraphDemoView::AssertValid() const
{
	CView::AssertValid();
}

void CGraphDemoView::Dump(CDumpContext& dc) const
{
	CView::Dump(dc);
}

CGraphDemoDoc* CGraphDemoView::GetDocument() // non-debug version is inline
{
	ASSERT(m_pDocument->IsKindOf(RUNTIME_CLASS(CGraphDemoDoc)));
	return (CGraphDemoDoc*) m_pDocument;
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CGraphDemoView message handlers

void CGraphDemoView::OnGraphCreate()
{
	// TODO: Add your command handler code here
	CCreateGraphDlg dlg(m_Graph, this) ;
	if (dlg.DoModal()  == IDOK) 
		m_NumNodes = dlg.m_NumNodes ;
}

void CGraphDemoView::OnGraphOptimize()
{
	// TODO: Add your command handler code here
	COptimizeDlg dlg(m_NumNodes, this) ;
	if (dlg.DoModal() == IDOK) {
		delete pTheGraphGADriver ;                           
		pTheGraphGADriver = new CGAGraphDriver(m_NumNodes,dlg.m_NumCols,dlg.m_NumRows)  ;
		pTheGraphGADriver->SetGraph(m_Graph) ;
		HCURSOR hcurSave;
		hcurSave = SetCursor(LoadCursor(NULL, IDC_WAIT));
		m_Evolving = TRUE ;
		pTheGraphGADriver->Optimize(dlg.m_NumGen) ;
		SetCursor(hcurSave);
		m_Evolving = FALSE ;
		m_View = GV_OPTIMIZED ;
		Invalidate() ;
		}
}

void CGraphDemoView::OnGraphViewOptimized()
{
	// TODO: Add your command handler code here
	if (pTheGraphGADriver != NULL) {
		m_View = GV_OPTIMIZED ;
		Invalidate()  ;
		}
}

void CGraphDemoView::OnGraphViewUnoptimized()
{
	// TODO: Add your command handler code here
	if (pTheGraphGADriver != NULL) {
		m_View = GV_UNOPTIMIZED ;
		Invalidate()  ;
		}
}

void CGraphDemoView::OnFileNew()
{
	// TODO: Add your command handler code here
	delete pTheGraphGADriver ;                           
	pTheGraphGADriver = NULL ;
	m_View = GV_NONE ;
	m_NumNodes = 0 ; 
	m_View = GV_NONE ;
	m_Evolving = FALSE;
	Invalidate()  ;
}

void CGraphDemoView::OnUpdateGraphCreate(CCmdUI* pCmdUI)
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(!m_Evolving) ;
}

void CGraphDemoView::OnUpdateGraphOptimize(CCmdUI* pCmdUI)
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(!m_Evolving && m_NumNodes > 0 ) ;
}

void CGraphDemoView::OnGraphStop()
{
	// TODO: Add your command handler code here
	pTheGraphGADriver->Stop() ;
}

void CGraphDemoView::OnUpdateGraphStop(CCmdUI* pCmdUI)
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(m_Evolving) ;		
}

void CGraphDemoView::OnUpdateGraphViewOptimized(CCmdUI* pCmdUI)
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(!m_Evolving && m_NumNodes > 0 && pTheGraphGADriver) ;		
	pCmdUI->SetCheck(m_View == GV_OPTIMIZED) ;		
}

void CGraphDemoView::OnUpdateGraphViewUnoptimized(CCmdUI* pCmdUI)
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(!m_Evolving && m_NumNodes > 0 && pTheGraphGADriver) ;		
	pCmdUI->SetCheck(m_View == GV_UNOPTIMIZED) ;		
}

void CGraphDemoView::OnUpdateFileNew(CCmdUI* pCmdUI)
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(!m_Evolving) ;		
}

void CGraphDemoView::OnUpdateAppExit(CCmdUI* pCmdUI)
{
	// TODO: Add your command update UI handler code here
	pCmdUI->Enable(!m_Evolving) ;		
}
